#!/usr/bin/env python3
# Phytec 2022, V1.0

import sys
import os
import json
import time
import gi
gi.require_version('Gst', '1.0')
from gi.repository import Gst, GObject
from argparse import ArgumentParser
from pprint import pprint

import isp_json


def read_aec_enable(isp):
    request = {
        'id': 'ae.g.en',
        'streamid': 0,
    }

    data = isp.read_json(request)

    return data['enable']


def write_aec_enable(isp, enable=True):
    request = {
        'enable': enable,
        'id': 'ae.s.en',
        'streamid': 0,
    }

    isp.write_json(request)


def read_awb_enable(isp):
    request = {
        'id': 'awb.g.en',
        'streamid': 0,
    }

    data = isp.read_json(request)

    return data['enable']


def write_awb_enable(isp, enable=True):
    request = {
        'enable': enable,
        'id': 'awb.s.en',
        'streamid': 0,
    }

    isp.write_json(request)


def get_wb(isp):
    request = {
        'id': 'wb.g.cfg',
        'streamid': 0,
    }

    return isp.read_json(request)


def set_wb(isp, red=1.0, green=1.0, blue=1.0):
    request = {
        'matrix': [1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 1.0],
        'offset': {
            'blue': 0,
            'green': 0,
            'red': 0,
        },
        'wb.gains': {
            'blue': blue,
            'green.b': green,
            'green.r': green,
            'red': red,
        },
        'id': 'wb.s.cfg',
        'streamid': 0,
    }

    isp.write_json(request)


def read_dwe_bypass(isp):
    request = {
        'id': 'dwe.g.params',
        'streamid': 0,
    }

    data = isp.read_json(request)

    return data['dwe']['bypass']


def write_dwe_bypass(isp, enable=True):
    request = {
        'id': 'dwe.s.bypass',
        'streamid': 0,
        'dwe': {
            'bypass': enable,
        }
    }

    isp.write_json(request)


def sensor_query(isp):
    request = {
        'id': 'sensor.query',
        'streamid': 0,
    }

    data = isp.read_json(request)

    return data


def handle_awb(isp, state='on', red=1.0, green=1.0, blue=1.0):
    if state == 'on':
        write_awb_enable(isp, True)
    elif state == 'off':
        write_awb_enable(isp, False)
    elif state == 'manual':
        write_awb_enable(isp, False)
        set_wb(isp, red=red, green=green, blue=blue)
    else:
        pprint(json.dumps(get_wb(isp), indent=4))


def handle_aec(isp, state='on'):
    if state == 'on':
        write_aec_enable(isp, True)
    else:
        write_aec_enable(isp, False)


def handle_dwe(isp, state='on'):
    if state == 'on':
        write_dwe_bypass(isp, False)
    else:
        write_dwe_bypass(isp, True)


def toggle_features(isp, waittime=5.0, red=1.0, green=1.0, blue=1.0):

    Gst.init(sys.argv)

    pipeline = Gst.Pipeline()

    src = Gst.ElementFactory.make('v4l2src')
    src.set_property('device', '/dev/video-csi1')
    sink = Gst.ElementFactory.make('waylandsink')
    sink.set_property('sync', False)
    textoverlay = Gst.ElementFactory.make('textoverlay')
    textoverlay.set_property('font-desc', 'Sans, 14')
    caps = Gst.Caps.from_string('video/x-raw,width=1920,height=1080')

    pipeline.add(src)
    pipeline.add(sink)
    pipeline.add(textoverlay)
    src.link_filtered(textoverlay, caps)
    textoverlay.link(sink)
    pipeline.set_state(Gst.State.PLAYING)

    write_dwe_bypass(isp, False)
    write_aec_enable(isp, True)
    write_awb_enable(isp, True)

    textoverlay.set_property('text', 'Dewarp:  ON  AWB:  ON  AEC:  ON')

    while 1:
        try:
            time.sleep(waittime)
            textoverlay.set_property('text', 'Dewarp: OFF  AWB:  ON  AEC:  ON')
            write_dwe_bypass(isp, True)
            time.sleep(waittime)
            textoverlay.set_property('text', 'Dewarp: OFF  AWB: OFF  AEC:  ON')
            write_awb_enable(isp, False)
            set_wb(isp, red=red, green=green, blue=blue)
            time.sleep(waittime)
            textoverlay.set_property('text', 'Dewarp: OFF  AWB: OFF  AEC: OFF')
            write_aec_enable(isp, False)
            time.sleep(waittime)


            write_aec_enable(isp, True)
            time.sleep(0.5)
            textoverlay.set_property('text', 'Dewarp:  OFF  AWB:  OFF  AEC:  ON')
            time.sleep(waittime)
            write_awb_enable(isp, True)
            textoverlay.set_property('text', 'Dewarp:  OFF  AWB:  ON  AEC: ON')
            time.sleep(waittime)
            write_dwe_bypass(isp, False)
            textoverlay.set_property('text', 'Dewarp:  ON  AWB:  ON  AEC:  ON')

        except KeyboardInterrupt:
            break

    pipeline.set_state(Gst.State.NULL)


def main(args):
    parser = ArgumentParser(description='PHYTEC i.MX 8MP ISP feature Demo')
    parser.add_argument('-t', '--time', type=float, default=5.0,
                        help='Waiting time in seconds')
    parser.add_argument('-r', type=float, dest='red',
                        help='red gain (manual)', default=1.0)
    parser.add_argument('-b', type=float, dest='blue',
                        help='blue gain (manual)', default=1.0)
    parser.add_argument('-g', type=float, dest='green',
                        help='green gain (manual)', default=1.0)

    args = parser.parse_args()

    vd = os.open('/dev/video-csi1', os.O_RDWR | os.O_NONBLOCK, 0)
    isp = isp_json.IspJson(vd)

    toggle_features(isp, waittime=args.time,
                    red=args.red,
                    green=args.green,
                    blue=args.blue)

    os.close(vd)


if __name__ == '__main__':
    sys.exit(main(sys.argv))
